@@ -1,6 +1,6 @@
 package Resque::Job;
 # ABSTRACT: Resque job container
-$Resque::Job::VERSION = '0.30';
+$Resque::Job::VERSION = '0.31';
 use Moose;
 use Moose::Util::TypeConstraints;
 with 'Resque::Encoder';
@@ -21,33 +21,26 @@ has worker  => (
     predicate => 'has_worker'
 );
 
-has class   => ( is => 'rw', lazy => 1, default => sub { confess "This job needs a class to do some work." } );
+has class => ( is => 'rw', lazy => 1, default => sub { confess "This job needs a class to do some work." } );
 
-has queue   => (
+has queue => (
     is        => 'rw', lazy => 1,
     default   => \&queue_from_class,
     predicate => 'queued'
 );
 
-has args    => ( is => 'rw', isa => 'ArrayRef', default => sub {[]} );
+has args => ( is => 'rw', isa => 'ArrayRef', default => sub {[]} );
 
 coerce 'HashRef'
     => from 'Str'
     => via { JSON->new->utf8->decode($_) };
 has payload => (
-    is   => 'ro',
-    isa  => 'HashRef',
-    coerce => 1,
-    lazy => 1,
-    default => sub {{
-        class => $_[0]->class,
-        args  => $_[0]->args
-    }},
-    trigger => sub {
-        my ( $self, $hr ) = @_;
-        $self->class( $hr->{class} );
-        $self->args( $hr->{args} ) if $hr->{args};
-    }
+    is      => 'ro',
+    isa     => 'HashRef',
+    coerce  => 1,
+    lazy    => 1,
+    builder => 'payload_builder',
+    trigger => \&_payload_trigger
 );
 
 sub encode {
@@ -113,6 +106,19 @@ sub fail {
     );
 }
 
+sub payload_builder {+{
+    class => $_[0]->class,
+    args  => $_[0]->args
+}}
+
+sub payload_reader {
+    my ( $self, $hr ) = @_;
+    $self->class( $hr->{class} );
+    $self->args( $hr->{args} ) if $hr->{args};
+}
+
+sub _payload_trigger { shift->payload_reader(@_) }
+
 __PACKAGE__->meta->make_immutable();
 
 __END__
@@ -127,7 +133,7 @@ Resque::Job - Resque job container
 
 =head1 VERSION
 
-version 0.30
+version 0.31
 
 =head1 ATTRIBUTES
 
@@ -217,6 +223,19 @@ Store a failure (or exception and failure) on this job.
     $job->fail( "error message'); # or
     $job->fail( ['exception', 'error message'] );
 
+=head2 payload_builder
+
+Default payload builder method. This method is public only to be wrapped in the
+context of plugins that adds attributes to this class.
+
+=head2 payload_reader
+
+Default payload trigger method. This method is public only to be wrapped in the
+context of plugins that adds attributes to this class.
+
+This mehtod is only called at construction time to populate job class and args
+attributes from payload.
+
 =head1 AUTHOR
 
 Diego Kuperman <diego@freekeylabs.com>
